!
! bandrm --- this subroutine reduces a real symmetric band matrix
!       to a symmetric tridiagonal matrix
!
!
!     THIS SUBROUTINE IS A MODIFIED VERSION OF BANDR
!     THE MATRIX A IS IN BAND STORRAGE MODE AND MUST BE DIMENSIONED
!     EXACTELY AS N BY MB. NM DENOTES THE NUMBER OF ROWS OF THE
!     EIGENVECTOR MATRIX WHICH SHALL BE COMPUTED.
!     ONE COMPUTES THE FIRST NM/2 AND THE LAST NM/2 ROWS.
!
      subroutine bandrm(NM,N,MB,A,D,E,E2,MATZ,Z)
      use fortrantl
      implicit none

      INTEGER J,K,L,N,R,I1,I2,J1,J2,KR,MB,MR,M1,NM,N2,R1,UGL,MAXL,MAXR
      integer nmn, jj
      REAL(dblprec) A(N,MB),D(N),E(N),E2(N),Z(NM,N)
      REAL(dblprec) G,U,B1,B2,C2,F1,F2,S2,DMIN,DMINRT
      INTEGER MAX0,MIN0,MOD
      LOGICAL MATZ
!
!     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BANDRD,
!     NUM. MATH. 12, 231-241(1968) BY SCHWARZ.
!     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 273-283(1971).
!
!     THIS SUBROUTINE REDUCES A REAL SYMMETRIC BAND MATRIX
!     TO A SYMMETRIC TRIDIAGONAL MATRIX USING AND OPTIONALLY
!     ACCUMULATING ORTHOGONAL SIMILARITY TRANSFORMATIONS.
!
!     ON INPUT:
!
!        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
!          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
!          DIMENSION STATEMENT;
!
!        N IS THE ORDER OF THE MATRIX;
!
!        MB IS THE (HALF) BAND WIDTH OF THE MATRIX, DEFINED AS THE
!          NUMBER OF ADJACENT DIAGONALS, INCLUDING THE PRINCIPAL
!          DIAGONAL, REQUIRED TO SPECIFY THE NON-ZERO PORTION OF THE
!          LOWER TRIANGLE OF THE MATRIX;
!
!        A CONTAINS THE LOWER TRIANGLE OF THE SYMMETRIC BAND INPUT
!          MATRIX STORED AS AN N BY MB ARRAY.  ITS LOWEST SUBDIAGONAL
!          IS STORED IN THE LAST N+1-MB POSITIONS OF THE FIRST COLUMN,
!          ITS NEXT SUBDIAGONAL IN THE LAST N+2-MB POSITIONS OF THE
!          SECOND COLUMN, FURTHER SUBDIAGONALS SIMILARLY, AND FINALLY
!          ITS PRINCIPAL DIAGONAL IN THE N POSITIONS OF THE LAST COLUMN.
!          CONTENTS OF STORAGES NOT PART OF THE MATRIX ARE ARBITRARY;
!
!        MATZ SHOULD BE SET TO .TRUE. IF THE TRANSFORMATION MATRIX IS
!          TO BE ACCUMULATED, AND TO .FALSE. OTHERWISE.
!
!     ON OUTPUT:
!
!        A HAS BEEN DESTROYED, EXCEPT FOR ITS LAST TWO COLUMNS WHICH
!          CONTAIN A COPY OF THE TRIDIAGONAL MATRIX;
!
!        D CONTAINS THE DIAGONAL ELEMENTS OF THE TRIDIAGONAL MATRIX;
!
!        E CONTAINS THE SUBDIAGONAL ELEMENTS OF THE TRIDIAGONAL
!          MATRIX IN ITS LAST N-1 POSITIONS.  E(1) IS SET TO ZERO;
!
!        E2 CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E.
!          E2 MAY COINCIDE WITH E IF THE SQUARES ARE NOT NEEDED;
!
!        Z CONTAINS THE ORTHOGONAL TRANSFORMATION MATRIX PRODUCED IN
!          THE REDUCTION IF MATZ HAS BEEN SET TO .TRUE.  OTHERWISE, Z
!          IS NOT REFERENCED.
!
!     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
!     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
!
!     ------------------------------------------------------------------
!
      DMIN = 2.0D0**(-64)
      DMINRT = 2.0D0**(-32)
!     :::::::::: INITIALIZE DIAGONAL SCALING MATRIX ::::::::::
      DO 30 J = 1, N
        D(J) = 1.0D0
   30 continue

      IF (.NOT. MATZ) GO TO 60
!
!.....HERE Z(J,K) IS INITIALIZED IN A SPECIAL FORM.
!
      NMN = NM/2
      DO 50 J = 1, NMN
         DO 40 K = 1, N
           Z(J,K) = 0.0D0
   40    continue
         Z(J,J) = 1.0D0
   50 CONTINUE

      DO 51 J = NMN+1, NM
         DO 41 K = 1, N
           Z(J,K)  = 0.D0
   41    continue
         JJ = J + N - NM
         Z(J,JJ) = 1.D0
   51    CONTINUE
!
!.....END OF SPECIAL CHANGES
!
   60 M1 = MB - 1
! CB
!      IF (M1 - 1) 900, 800, 70
      if ((M1 - 1) < 0) then
        goto 900
      else if ((M1 - 1) == 0) then
        goto 800
      else
        goto 70
      end if
! end CB
   70 N2 = N - 2

      DO 700 K = 1, N2
         MAXR = MIN0(M1,N-K)
!     :::::::::: FOR R=MAXR STEP -1 UNTIL 2 DO -- ::::::::::
         DO 600 R1 = 2, MAXR
            R = MAXR + 2 - R1
            KR = K + R
            MR = MB - R
            G = A(KR,MR)
            A(KR-1,1) = A(KR-1,MR+1)
            UGL = K

            DO 500 J = KR, N, M1
               J1 = J - 1
               J2 = J1 - 1
               IF (G .EQ. 0.0D0) GO TO 600
               B1 = A(J1,1) / G
               B2 = B1 * D(J1) / D(J)
               S2 = 1.0D0 / (1.0D0 + B1 * B2)
               IF (S2 .GE. 0.5D0 ) GO TO 450
               B1 = G / A(J1,1)
               B2 = B1 * D(J) / D(J1)
               C2 = 1.0D0 - S2
               D(J1) = C2 * D(J1)
               D(J) = C2 * D(J)
               F1 = 2.0D0 * A(J,M1)
               F2 = B1 * A(J1,MB)
               A(J,M1) = -B2 * (B1 * A(J,M1) - A(J,MB)) - F2 + A(J,M1)
               A(J1,MB) = B2 * (B2 * A(J,MB) + F1) + A(J1,MB)
               A(J,MB) = B1 * (F2 - F1) + A(J,MB)

               DO 200 L = UGL, J2
                  I2 = MB - J + L
                  U = A(J1,I2+1) + B2 * A(J,I2)
                  A(J,I2) = -B1 * A(J1,I2+1) + A(J,I2)
                  A(J1,I2+1) = U
  200          CONTINUE

               UGL = J
               A(J1,1) = A(J1,1) + B2 * G
               IF (J .EQ. N) GO TO 350
               MAXL = MIN0(M1,N-J1)

               DO 300 L = 2, MAXL
                  I1 = J1 + L
                  I2 = MB - L
                  U = A(I1,I2) + B2 * A(I1,I2+1)
                  A(I1,I2+1) = -B1 * A(I1,I2) + A(I1,I2+1)
                  A(I1,I2) = U
  300          CONTINUE

               I1 = J + M1
               IF (I1 .GT. N) GO TO 350
               G = B2 * A(I1,1)
  350          IF (.NOT. MATZ) GO TO 500

               DO 400 L = 1, NM
                  U = Z(L,J1) + B2 * Z(L,J)
                  Z(L,J) = -B1 * Z(L,J1) + Z(L,J)
                  Z(L,J1) = U
  400          CONTINUE

               GO TO 500

  450          U = D(J1)
               D(J1) = S2 * D(J)
               D(J) = S2 * U
               F1 = 2.0D0 * A(J,M1)
               F2 = B1 * A(J,MB)
               U = B1 * (F2 - F1) + A(J1,MB)
               A(J,M1) = B2 * (B1 * A(J,M1) - A(J1,MB)) + F2 - A(J,M1)
               A(J1,MB) = B2 * (B2 * A(J1,MB) + F1) + A(J,MB)
               A(J,MB) = U

               DO 460 L = UGL, J2
                  I2 = MB - J + L
                  U = B2 * A(J1,I2+1) + A(J,I2)
                  A(J,I2) = -A(J1,I2+1) + B1 * A(J,I2)
                  A(J1,I2+1) = U
  460          CONTINUE

               UGL = J
               A(J1,1) = B2 * A(J1,1) + G
               IF (J .EQ. N) GO TO 480
               MAXL = MIN0(M1,N-J1)

               DO 470 L = 2, MAXL
                  I1 = J1 + L
                  I2 = MB - L
                  U = B2 * A(I1,I2) + A(I1,I2+1)
                  A(I1,I2+1) = -A(I1,I2) + B1 * A(I1,I2+1)
                  A(I1,I2) = U
  470          CONTINUE

               I1 = J + M1
               IF (I1 .GT. N) GO TO 480
               G = A(I1,1)
               A(I1,1) = B1 * A(I1,1)
  480          IF (.NOT. MATZ) GO TO 500

               DO 490 L = 1, NM
                  U = B2 * Z(L,J1) + Z(L,J)
                  Z(L,J) = -Z(L,J1) + B1 * Z(L,J)
                  Z(L,J1) = U
  490          CONTINUE

  500       CONTINUE

  600    CONTINUE

         IF (MOD(K,64) .NE. 0) GO TO 700
!     :::::::::: RESCALE TO AVOID UNDERFLOW OR OVERFLOW ::::::::::
         DO 650 J = K, N
            IF (D(J) .GE. DMIN) GO TO 650
            MAXL = MAX0(1,MB+1-J)

            DO 610 L = MAXL, M1
              A(J,L) = DMINRT * A(J,L)
  610       continue

            IF (J .EQ. N) GO TO 630
            MAXL = MIN0(M1,N-J)

            DO 620 L = 1, MAXL
               I1 = J + L
               I2 = MB - L
               A(I1,I2) = DMINRT * A(I1,I2)
  620       CONTINUE

  630       IF (.NOT. MATZ) GO TO 645

            DO 640 L = 1, NM
              Z(L,J) = DMINRT * Z(L,J)
  640       continue

  645       A(J,MB) = DMIN * A(J,MB)
            D(J) = D(J) / DMIN
  650    CONTINUE

  700 CONTINUE
!     :::::::::: FORM SQUARE ROOT OF SCALING MATRIX ::::::::::
  800 DO 810 J = 2, N
        E(J) = DSQRT(D(J))
  810 continue

      IF (.NOT. MATZ) GO TO 840

      DO 830 J = 1, NM

         DO 820 K = 2, N
           Z(J,K) = E(K) * Z(J,K)
  820    continue

  830 CONTINUE

  840 U = 1.0D0

      DO 850 J = 2, N
         A(J,M1) = U * E(J) * A(J,M1)
         U = E(J)
         E2(J) = A(J,M1) ** 2
         A(J,MB) = D(J) * A(J,MB)
         D(J) = A(J,MB)
         E(J) = A(J,M1)
  850 CONTINUE

      D(1) = A(1,MB)
      E(1) = 0.0D0
      E2(1) = 0.0D0
      GO TO 1001

  900 DO 950 J = 1, N
         D(J) = A(J,MB)
         E(J) = 0.0D0
         E2(J) = 0.0D0
  950 CONTINUE

 1001 RETURN
!     :::::::::: LAST CARD OF BANDR ::::::::::
      end subroutine bandrm