!
! woods --- plot Nimrods Woods-Saxonish path
! Christian Buth
! 9 May 2006
!


        program woods
        use fortrantl
        use SESpath
        implicit none

        integer i
        real(dblprec) r, rstep
        complex(dblprec) val

!         call initSESpath(0.13D0, 5.0D0, 7.0D0)
        rstep = 40.0 / 200.0

        do i=1, 200
          r = -20.0 + (i-1) * rstep
!          val = nimrodws(r)
!          val = robinme(r)
!           val = woodsax(r)
          val = karlssonwoodsax(r)
!           val = pathFc(r)
!           write (*,*) real(val), aimag(val), r
          write (*,*) real(val), aimag(val)
        end do

!         call exitSESpath


        contains

! 		
! nimrodws --- Nimrods Moiseyev's smooth exterior scaling path
! 

        complex(dblprec) function nimrodws(r)
        use fortrantl
        implicit none

        real(dblprec) r, lambda, r0, theta
        complex(dblprec) pf

        theta = 0.75
        pf = exp( (0,1) * cmplx(theta) ) - (1, 0)
        lambda = 5.0
        r0 = 6

        nimrodws = r + pf * (r + 0.5 / lambda * log(  &
       &        cosh(lambda * (r-r0))                 &
       &      / cosh(lambda * (r+r0)) ))
        end function nimrodws


! 		
! woodsax --- Nimrod Moiseyev's path expressed in terms of 
! 	exponential functions
! 

        complex(dblprec) function woodsax(r)
        use fortrantl
        implicit none

        real(dblprec) r, lambda, r0, theta
        complex(dblprec) pf

        theta = 0.75
        pf = exp( (0,1) * cmplx(theta) ) - (1, 0)
        lambda = 5.0
        r0 = 6

        woodsax = r + pf * (r + 0.5 / lambda * ( log(  &
       &             (1.0 + exp(2*lambda * (r-r0)))    &
       &           / (1.0 + exp(2*lambda * (r+r0))) )  &
       &           + 2.0 * lambda * r0 ))
        end function woodsax


! 
! robinme --- try to leave out half of the path	
! 
	
        complex(dblprec) function robinme(r)
        use fortrantl
        implicit none

        real(dblprec) r, lambda, r0, theta
        complex(dblprec) pf

        theta = 0.75
        pf = exp( (0,1) * cmplx(theta) ) - (1, 0)
        lambda = 5.0
        r0 = 6

        robinme = r + pf * (r + 0.5 / lambda * ( log(  &
       &           (1.0 + exp(2*lambda * (r-r0))) )    &
       &           + 2.0 * lambda * r0 ))
        end function robinme


! 		
! karlssonwoodsax --- Nimrod Moiseyev's path expressed in terms 
! 	of exponential functions as given by Karlsson in J. Chem.
! 	Phys, *109* 9366 (1998)
! 

        complex(dblprec) function karlssonwoodsax(r)
        use fortrantl
        implicit none

        real(dblprec) r, lambda, r0, theta
        complex(dblprec) pf

        theta = 0.75
        pf = exp( (0,1) * cmplx(theta) ) - (1, 0)
        lambda = 5.0
        r0 = 6

        karlssonwoodsax = r + pf * (r + log(         &
       &             (1.0 + exp(2*lambda * (r0+r)))  &
       &           / (1.0 + exp(2*lambda * (r0-r))) ))
        end function karlssonwoodsax
        end program woods
