!
! floquetmat --- routines to generate the Floquet matrix
! Christian Buth
! 12 June 2006
!


      module floquetmat
      use fortrantl
      implicit none


!
! Interface
!

      interface
      subroutine readatom(nl, lmin, lmax, nvec, nvlnc, eval,      &
     &     SESdipatm, SESfTfatm, SESpotatm, SESWatom, li, init, Eground)
      use fortrantl
      implicit none

      integer, intent(in) :: nl, lmin, lmax, nvec, li, init
      integer, intent(out) :: nvlnc
      real(dblprec), intent(out) :: eval(nvec,lmin:lmax), Eground
      complex(dblprec), intent(out) ::                                &
     &          SESWatom(nvec,nvec,lmin:lmax),                        &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),             &
     &          SESfTfatm(nvec,nvec,lmin:lmax),                       &
     &          SESpotatm(nvec,nvec,lmin:lmax)
      end subroutine readatom


      subroutine hamiltonian(dimen, nmain, nphmin, nphmax, lmin, lmax, nvec, mi,  &
     &          cconst, eval, Eground, omega, SESfTfatm, SESpotatm,        &
     &          SESWatom, SESdipatm, ceval)
      use matrixfile
      implicit none

      integer, intent(in) :: dimen, nmain, nphmin, nphmax, lmin, lmax, nvec, mi
      real(dblprec), intent(in) :: cconst
      real(dblprec), intent(in) :: eval(nvec,lmin:lmax)
      real(dblprec), intent(in) :: Eground, omega
      complex(dblprec), intent(in) :: SESfTfatm(nvec,nvec,lmin:lmax),  &
     &                                SESpotatm(nvec,nvec,lmin:lmax),  &
     &                                SESWatom (nvec,nvec,lmin:lmax),  &
     &                                SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax)
      complex(dblprec), intent(in), optional :: ceval(nvec,lmin:lmax)
      end subroutine hamiltonian


      subroutine fulldiagfloquet(jobz, dimen, ceig, cvec)
      use fortrantl
      implicit none

      character, intent(in) :: jobz
      integer, intent(in) :: dimen
      complex(dblprec), intent(out) :: ceig(dimen)
      complex(dblprec), optional, intent(out) :: cvec(dimen,dimen)
      end subroutine fulldiagfloquet


      subroutine lanczosdiagfloquet(ceig, polest, nmat, nmain, niter, istart, admixture)
      use fortrantl
      implicit none

      integer, intent(in) :: nmat, nmain, niter, istart(nmain)
      real(dblprec), intent(in) :: admixture
      real(dblprec), intent(out) :: polest(niter)
      complex(dblprec), intent(out) :: ceig(niter)
      end subroutine lanczosdiagfloquet
      end interface
      end module floquetmat
