SUBROUTINE Dmtrx(D,x)
  USE FiniteElements
  implicit none

!! Dmtrx calculates the dipole matrix

  REAL(kind=8), intent(OUT) :: D(-5:5,*)
  TYPE(FEBnodes) :: x
  REAL(kind=8) :: smO(0:2,0:2), opO(0:2,0:2), smD(0:2,0:2), opD(0:2,0:2)
  INTEGER :: nmnds, nbasis, left, right, row1, j, k, l
  REAL(kind=8) :: h1, h2
  REAL(kind=8), ALLOCATABLE :: nodes(:)
  
  nmnds = x%numnds
  nbasis = numbasfun(x)

  ALLOCATE( nodes(nmnds) )

  nodes = x%nodes
  left = x%numleft
  right = x%numright


  if (nbasis <= 0) RETURN
  D(-5:5,1:nbasis) = 0.d0

! Overlap for half-basis functions "centered" on same node
  smO(0:2,0:2) = RESHAPE( SOURCE = (/ &
    & 181.d0/462.d0, -311.d0/4620.d0, 281.d0/55440.d0, &
    &-311.d0/4620.d0, 52.d0/3465.d0, -23.d0/18480.d0, &
    & 281.d0/55440.d0, -23.d0/18480.d0, 1.d0/9240.d0 &
    & /), SHAPE = (/3, 3/) )

! Overlap for half-basis functions "centered" on adjacent nodes
  opO(0:2,0:2) = RESHAPE( SOURCE = (/ &
    &  25.d0/231.d0, 151.d0/4620.d0, 181.d0/55440.d0, &
    &-151.d0/4620.d0, -19.d0/1980.d0, -13.d0/13860.d0, &
    & 181.d0/55440.d0, 13.d0/13860.d0, 1.d0/11088.d0 &
    & /), SHAPE = (/3, 3/) )

! Dipole matrix for half-basis functions "centered" on same node
  smD(0:2,0:2) = RESHAPE( SOURCE = (/ &
    & 10.d0/33.d0, -23.d0/495.d0, 13.d0/3960.d0 , &
    &-23.d0/495.d0, 19.d0/1980.d0, -1.d0/1320.d0, &
    & 13.d0/3960.d0, -1.d0/1320.d0, 1.d0/15840.d0 &
    & /), SHAPE = (/3, 3/) )  

! Dipole matrix for half-basis functions "centered" on adjacent nodes
  opD(0:2,0:2) = RESHAPE( SOURCE = (/ &
    & 25.d0/462.d0, 239.d0/13860.d0, 1.d0/560.d0, &
    &-107.d0/6930.d0, -19.d0/3960.d0, -3.d0/6160.d0, &
    & 41.d0/27720.d0, 5.d0/11088.d0, 1.d0/22176.d0 &
    & /), SHAPE = (/3, 3/) )



!! half-basis functions "centered" at nodes(1)


    row1 = left-2
    h2 = node_spacing(x,1)
   
    DO j = (3-left),2
      DO k = j,2
        D(j-k,row1+k) = -(-h2)**(j+k+2)*smD(j,k)-nodes(2)*(-h2)**(j+k+1)*smO(j,k)
       END DO
    END DO

!! overlaps between nodes(1) and nodes(2)


   h1 = h2
   IF (nmnds > 2) THEN
     DO j = (3-left),2
       DO k = 0,2
          D(j-k-3,row1+3+k) = h1**(j+k+2)*opD(j,k)
       END DO
     END DO
   ELSE
     DO j = (3-left),2
       DO k = (3-right),2
         D(j-k-left,row1+left+k) = h1**(j+k+2)*opD(j,k)
       END DO
     END DO
   ENDIF

 
  h1 = h2

!! basis functions centered at nodes(2)
   IF (nmnds > 2) THEN
     row1 = left + 1
     h2 = node_spacing(x,2)

     DO j = 0,2
       DO k = j,2
         D(j-k,row1+k) = ( (h1**(j+k+2) - (-h2)**(j+k+2) )*smD(j,k) &
         & +( nodes(1)*(h1**(j+k+1)) - nodes(3)*(-h2)**(j+k+1) )*smO(j,k) )
       END DO
     END DO
   ENDIF



   DO l = 3,(nmnds-1)

!! overlaps between nodes(l-1) and nodes(l)
    h1 = h2
    DO j = 0,2
      DO k = 0,2
        D(j-k-3,row1+3+k) = h1**(j+k+2)*opD(j,k) + h1**(j+k+1)*nodes(l-1)*opO(j,k)
      END DO
    END DO



!! basis functions centered on nodes(l)
    row1 = row1 + 3
    h2 = node_spacing(x,l)
    DO j = 0,2
      DO k = j,2
        D(j-k,row1+k) = ( (h1**(j+k+2) - (-h2)**(j+k+2) )*smD(j,k) &
        & +( nodes(l-1)*(h1**(j+k+1)) - nodes(l+1)*(-h2)**(j+k+1) )*smO(j,k) )
      END DO
    END DO
  END DO



!! overlaps between nodes(nmnds-1) and nodes(nmnds)
  IF (nmnds > 2) THEN
    h1 = h2
    DO j = 0,2
      DO k = (3-right),2
        D(j-k-right,row1+right+k) = h1**(j+k+2)*opD(j,k) + h1**(j+k+1)*nodes(nmnds-1)*opO(j,k)
      END DO
    END DO
  END IF



!! half-basis functions "centered" on nodes(nmnds)
    row1 = row1 + right
    DO j = (3-right),2
      DO k = j,2
        D(j-k,row1+k) = ( (h1**(j+k+2) )*smD(j,k) &
        & +( nodes(nmnds-1)*(h1**(j+k+1)))*smO(j,k)) 
      END DO
    END DO

    DO k = 1,(nbasis-1)
      DO j = (k+1),MIN(nbasis,k+5)
        D(j-k,k) = D(k-j,j)
      END DO
    END DO

  RETURN
END SUBROUTINE Dmtrx
