
SUBROUTINE Vmtrx(V,ll,rr,rV,b,c,d,x)
  USE FiniteElements

  implicit none

  TYPE(FEBnodes) :: x
  REAL(kind=8) :: V(-5:5,*), ll, rr(441), rV(441), b(441), c(441), d(441)
  INTEGER :: nmnds, nbasis, left, right, gnm, row1, j, k, l, mu
  REAL(kind=8), ALLOCATABLE :: f(:), r(:), gx(:), gw(:), q(:,:), p(:,:)
  REAL(kind=8) :: tol, h1, h2
  REAL(kind=8) :: seval

  nmnds = x%numnds
  left = x%numleft
  right = x%numright

!!numbasfun is defined in FiniteElements.f90 and gives the total # of basis functions
!!nbasis will be the size (nbasis*nbasis) of the full V matrix 
  
  nbasis = numbasfun(x)
  IF (nbasis <= 0) THEN
    RETURN
  ENDIF
  V(-5:5,1:nbasis) = 0.d0

  tol = 1.D-5

  gnm = 20  !!This is the amount of points to be used by the Gaussian integration
  
  ALLOCATE ( r(gnm), f(gnm), gx(gnm), gw(gnm), q(gnm,0:2), p(gnm,0:2) )
 
  CALL gauleg(-1.d0, 1.d0, gx, gw, gnm)

!!Coordinate transformation for Gaussian integration on interval (0,1)
  
   
  gx(:) = (gx(:) + 1.d0)/2.d0
  gw(:) = gw(:)/2.d0

  r(:) = gx(:)

  DO mu = 0,2
    q(:,mu) = FEbasfun(r,mu)
  END DO
  r(:) = r(gnm:1:-1)
  DO mu = 0,2
    p(:,mu) = ((-1)**mu)*FEbasfun(r,mu);
  END DO

! half-basis functions "centered" at nodes(1)
  row1 = left-2
  h2 = node_spacing(x,1)
  r(:) = h2*gx(:) + x%nodes(1)
  DO j = 1, gnm
!     f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/(cosh(r(j)))**2 - tanh(r(j))/r(j)
!     f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/r(j)
!     f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 4.d0/r(j) + 7.d0*dexp(-6.d0*(r(j)-6.d0)**2)
!     f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + 7.5d0*r(j)**2*dexp(-r(j))
     f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + seval(441,r(j),rr,rV,b,c,d)/r(j)
  END DO

  DO j = (3-left),2
    DO k = j,2
      V(j-k,row1+k) = (h2**(j+k+1))*SUM(gw(:)*p(:,j)*p(:,k)*f(:))
    END DO
  END DO


! overlaps between nodes(1) and nodes(2)
  h1 = h2
  IF (nmnds > 2) THEN
    DO j = (3-left),2
      DO k = 0,2
        V(j-k-3,row1+3+k) = (h1**(j+k+1))*sum(gw(:)*p(:,j)*q(:,k)*f(:))
      END DO
    END DO
  ELSE
    DO j = (3-left),2
      DO k = (3-right),2
        V(j-k-left,row1+left+k) = (h1**(j+k+1))*sum(gw(:)*p(:,j)*q(:,k)*f(:));
      END DO
    END DO
  ENDIF

  h1 = h2;
! basis functions centered at nodes(2)
  IF (nmnds > 2) THEN
    row1 = left + 1
    DO j = 0,2
      DO k = j,2
        V(j-k,row1+k) = (h1**(j+k+1))*sum(gw(:)*q(:,j)*q(:,k)*f(:))
      END DO
    END DO
    h2 = node_spacing(x,2)
    r(:) = h2*gx(:) + x%nodes(2)
    DO j = 1, gnm
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/(cosh(r(j)))**2 - tanh(r(j))/r(j)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/r(j)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 4.d0/r(j) + 7.d0*dexp(-6.d0*(r(j)-6.d0)**2)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + 7.5d0*r(j)**2*dexp(-r(j))
       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + seval(441,r(j),rr,rV,b,c,d)/r(j)
    END DO
    DO j = 0,2
      DO k = j,2
        V(j-k,row1+k) = V(j-k,row1+k) + (h2**(j+k+1))*sum(gw(:)*p(:,j)*p(:,k)*f(:))
      END DO
    END DO
  ENDIF

  DO l = 3,(nmnds-1)
! overlaps between nodes(l-1) and nodes(l)
    h1 = h2
    DO j = 0,2
      DO k = 0,2
        V(j-k-3,row1+3+k) = (h1**(j+k+1))*sum(gw(:)*p(:,j)*q(:,k)*f(:))
      END DO
    END DO

  ! basis functions centered on nodes(l)
    row1 = row1 + 3;
    DO j = 0,2
      DO k = j,2
        V(j-k,row1+k) = (h1**(j+k+1))*sum(gw(:)*q(:,j)*q(:,k)*f(:))
      END DO
    END DO
    h2 = node_spacing(x,l)
    r(:) = h2*gx(:) + x%nodes(l)
    DO j = 1, gnm
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/(cosh(r(j)))**2 - tanh(r(j))/r(j)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 1.d0/r(j)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 - 4.d0/r(j) + 7.d0*dexp(-6.d0*(r(j)-6.d0)**2)
!       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + 7.5d0*r(j)**2*dexp(-r(j))
       f(j) = 0.5d0*ll*(ll+1.d0)/r(j)**2 + seval(441,r(j),rr,rV,b,c,d)/r(j)
    END DO
    DO j = 0,2
      DO k = j,2
        V(j-k,row1+k) = V(j-k,row1+k) + (h2**(j+k+1))*sum(gw(:)*p(:,j)*p(:,k)*f(:))
      END DO
    END DO
  END DO

! overlaps between nodes(nmnds-1) and nodes(nmnds)
  IF (nmnds > 2) THEN
    h1 = h2
    DO j = 0,2
      DO k = (3-right),2
        V(j-k-right,row1+right+k) = (h1**(j+k+1))*sum(gw(:)*p(:,j)*q(:,k)*f(:))
      END DO
    END DO
  ENDIF

! half-basis functions "centered" on nodes(nmnds)
  row1 = row1 + right
  DO j = (3-right),2
    DO k = j,2
      V(j-k,row1+k) = (h1**(j+k+1))*sum(gw(:)*q(:,j)*q(:,k)*f(:))
    END DO
  END DO

  DO k = 1,(nbasis-1)
    DO j = (k+1),MIN(nbasis,k+5)
      V(j-k,k) = V(k-j,j)
    END DO
  END DO

  RETURN
END SUBROUTINE Vmtrx
