SUBROUTINE eigenfxn( egvec, ngrd, rf, xh, nbasis, nstate, nl, xnd, egfxn )
 USE FiniteElements
 implicit none

INTEGER :: ngrd, nbasis, nstate, nl
INTEGER :: i, j, k
REAL(kind=8) :: egvec( nbasis, nstate, nl ), egfxn(ngrd, nstate, nl), rf 
REAL(kind=8), ALLOCATABLE ::  x(:), rr(:)
REAL(kind=8) :: xf, xh 
TYPE(FEBnodes) :: xnd
INTEGER :: it0, it1, it2, it3, ict

Print*,'Evaluating eigenfunctions'

Print*,'The number of grid points is',ngrd

!! Evaluate eigenfunctions on a grid rr
!! ngrd is the number of grid points
!! rf is the size of the grid
!! xh is the spacing of the even grid
!! nbasis is the number of basis functions
!! nstate is the number of eigenstates per l
!! nl is the number of l quantum numbers considered

CALL system_clock( it0, ict )

ALLOCATE( x(ngrd), rr(ngrd) )

!! Create square root mesh
  
   xf=dsqrt(rf)
   xh=xf/dble(ngrd)
   DO j = 1, ngrd
     x(j)=xh*dble(j)
     rr(j)=x(j)*x(j)
   END DO


egfxn(:,:,:) = 0.d0

!! columns of egfxn will contain the first few eigenfunctions
!! the input eigenvectors are the columns of egvec
!! fxneval evaluates, on a grid, a function given by finite element coefficients
 
DO k = 1, nl
   DO j = 1, nstate
      CALL fxneval1( egfxn(1,j,k), rr, ngrd, egvec(1,j,k), xnd )
!     do i = 1, ngrd
!        write(100+10*(j+1)+k-1,*) rr(i), egfxn(i,j,k)
!        write(200+10*(j+1)+k-1,*) rr(i), egfxn(i,j,k)/rr(i)
!     end do
   END DO
END DO

CALL system_clock(it1)
Print*,'Time is', ( (it1-it0)*1.d0/ict )

RETURN
END SUBROUTINE eigenfxn



