
!==========================================================================
!     matrix vector multiplication SUBROUTINE

!! n is the number of columns in the banded matrix
!! A is the banded matrix to be multiplied
!! v is the vector to be multiplied
!! w is the output
!! nrhs is the number of eigenvectors


SUBROUTINE Av_FEband(n, A, v, w, nrhs)
  implicit none
  INTEGER :: n, nrhs, j, ncol, md
  REAL(kind=8) :: v(n,nrhs), w(n,nrhs), A(-5:5,n)


  md = MIN(5,n-1)
  w(:,:) = 0.d0
  DO ncol = 1,nrhs
    DO j = 1,md
      w(1:(j+md),ncol) = w(1:(j+md),ncol) + v(j,ncol)*A((1-j):md,j)
    END DO

    DO j = md+1,n-md
      w((j-md):(j+md),ncol) = w((j-md):(j+md),ncol) + v(j,ncol)*A(-md:md,j)
    END DO

    DO j = MAX(md+1,n-md+1),n
      w((j-md):n,ncol) = w((j-md):n,ncol) + v(j,ncol)*A(-md:(n-j),j)
    END DO
  END DO

  RETURN
END SUBROUTINE Av_FEband


SUBROUTINE Av_FE2band(A,nb1,nb2,v,w,nrhs)
  implicit none
  INTEGER :: nb1, nb2, nrhs
  REAL(kind=8) :: v(nb1*nb2,nrhs), w(nb1*nb2,nrhs), A(-60:60,nb1*nb2)
  INTEGER :: col, blk, id, jd, kl, ku, strd1, strd2, j, d1, d2, diag, row, ncol

  d1 = min(5,nb1-1)
  d2 = min(5,nb2-1)
  kl = d2*nb1 + d1
  ku = kl
  strd1 = 2*d1 + 1
  strd2 = MAX(nb1,strd1)

  w(:,:) = 0.d0
  DO ncol = 1,nrhs
    DO col = 1, nb1*nb2
      DO blk = -d2,d2
        id = blk*strd2
        jd = blk*strd1
        DO j = -d1,d1
          diag = id + j
          row = diag + col
          IF ( (row>0).AND.(row<=nb1*nb2) ) THEN
            w(row,ncol) = w(row,ncol) + A(jd+j,col)*v(col,ncol)
          ENDIF
        END DO
      END DO
    END DO
  END DO

  RETURN
END SUBROUTINE Av_FE2band

